import { RunEnvironment } from "./enums";

/**
 * ## Description
 * In the context of a job, extendTimeout will increase the timeout of the currently executing payload.
 * This is useful when the payload is expected to take a long time. For example, when running pagination code.
 *
 * @example
 * ```typescript Example
 * import { extendTimeout } from "@intuned/sdk/runtime"
 *
 * extendTimeout();
 * ```
 *
 */
export declare function extendTimeout(): void;

/**
 * ## Description
 * In the context of a job or queue execution, extendPayload appends new payloads to the end of the queue of job.
 * Calling extendPayload will also call extendTimeout.
 *
 * @param {Payload | Payload[]} payload - The payload or array of payloads to extend. you can specify the api name and what parameters you want to pass it, the new added apis will use the same proxy and auth-session settings as the api that extended them
 *
 * @example
 * ```typescript Single payload
 * import { extendPayload } from "@intuned/sdk/runtime"
 *
 * // this function will append the exampleApi to the end of the queue or job it's executing in.
 * extendPayload({ api: 'exampleApi', parameters: { key: 'value' } });
 * ```
 *
 * @example
 * ```typescript Array of payloads
 * import { extendPayload } from "@intuned/sdk/runtime"
 *
 * const payloadArray: Payload[] = [
 *   { api: 'exampleApi1', parameters: { key1: 'value1' } },
 *   { api: 'exampleApi2', parameters: { key2: 'value2' } }
 * ];
 *
 * // this function will append 2 apis to the end of the queue or job it's executing in.
 * extendPayload(payloadArray);
 * ```
 */
export declare function extendPayload(payload: Payload | Payload[]): void;

/**
 * @interface Payload
 * @property {string} api - The API path you want to extend.
 * @property {Record<string, any>} parameters - A record of key-value pairs representing the parameters to be sent to the API
 *
 * @example
 *
 * ```typescript payload
 * import { Payload } from "@intuned/sdk/runtime"
 *
 * const payload: Payload = {
 *   api: 'exampleApi',
 *   parameters: {
 *     key1: 'value1',
 *     key2: 'value2'
 *   }
 * };
 * ```
 */
export interface Payload {
  api: string;
  parameters: Record<string, any>;
}

/**
 * Retrieves information about the current run environment.
 *
 * @returns {RunInfo} An object containing details about the run environment and the run ID.
 *
 * @example
 * ```typescript runInfo
 * import { runInfo } from "@intuned/sdk/runtime"
 *
 * const info = runInfo();
 * console.log(info.runEnvironment); // Outputs the run environment, IDE or DEPLOYED
 * console.log(info.runId); // Outputs the run ID, if available, in IDE run id will be undefined
 * ```
 */
export declare function runInfo(): RunInfo;

/**
 * Represents information about the current run.
 *
 * @interface RunInfo
 * @property {RunEnvironment} runEnvironment - the run environment `IDE` or `DEPLOYED`
 * @property {string} [runId] - Optional. The ID of the current run, in IDE environment, run id will be undefined
 */
export interface RunInfo {
  runEnvironment: RunEnvironment;
  runId?: string;
  jobId?: string;
  jobRunId?: string;
  queueId?: string;
  proxy?: string;
}

/**
 * @interface RunErrorOptions
 * @property {boolean} [retryable] - Optional. Indicates whether the error is retryable.
 * @property {number} [status_code] - Optional. The HTTP status code associated with the error.
 * @property {string} [error_code] - Optional. A specific error code to identify the type of error.
 *
 * @example
 * ```typescript RunErrorOptions
 * import { RunErrorOptions } from "@intuned/sdk/runtime"
 *
 * const options: RunErrorOptions = {
 *   retryable: true,
 *   status_code: 500,
 *   error_code: 'SERVER_ERROR'
 * };
 * ```
 */
export interface RunErrorOptions {
  retryable?: boolean;
  status_code?: number;
  error_code?: string;
}

/**
 * Represents an error that occurs during a run.
 *
 * @class
 * @extends Error
 *
 * @param {string} message - The error message.
 * @param {RunErrorOptions} [options] - Optional. Additional options for the error.
 *
 * @property {RunErrorOptions} options - The options associated with the error.
 *
 * @example
 * ```typescript RunError
 * import { RunError } from "@intuned/sdk/runtime"
 *
 * throw new RunError('An error occurred', {
 *   retryable: true,
 *   status_code: 500,
 *   error_code: 'SERVER_ERROR'
 * });
 *
 * ```
 */
export declare class RunError extends Error {
  constructor(message: string, options?: RunErrorOptions);
  options: RunErrorOptions;
}

/**
 * in the create auth session flow, you might to need a multiple choice answer from the user, requestMultipleChoice prompts the user with the question and possible options and returns their selection.
 *
 * **Note:** This function is currently in beta and may be subject to changes.
 *
 * @param {string} message - The message to display to the user.
 * @param {string[]} choices - An array of choices to present to the user.
 *
 * @example
 * ```typescript requestMultipleChoice
 * // in auth-sessions/create.ts
 *
 * import { requestMultipleChoice } from "@intuned/sdk/runtime"
 *
 * const message = "What is your favorite color?";
 * const choices = ["Red", "Blue", "Green", "Yellow"];
 *
 * const selectedChoice = yield requestMultipleChoice(message, choices);
 *
 * console.log(selectedChoice);
 * ```
 */
export declare function requestMultipleChoice(
  message: string,
  choices: string[]
): unknown;

/**
 * **Note:** This function is currently in beta and may be subject to changes.
 *
 * requestOTP help you to ask the user for an otp in the create auth-session flow.
 *
 * @param {string} message - The message to display to the user.
 *
 * @example
 * ```typescript requestOTP
 * // in auth-sessions/create.ts
 *
 * import { requestOTP } from "@intuned/sdk/runtime"
 *
 * const message = "please submit and OTP from your authenticator app";
 *
 * const otp = yield requestOTP(message);
 *
 * console.log(otp);
 * ```
 */
export declare function requestOTP(message: string): unknown;
